﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PropertyGridSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            property.SelectedObject = new Employee();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ToChangeDisplayName(property.SelectedObject, "Name", "Employee Name");
        }

        public void ToChangeDisplayName(object obj, string propertyName, string displayName)
        {
            
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(obj.GetType())[propertyName];
            DisplayNameAttribute attribute = (DisplayNameAttribute)
                                          descriptor.Attributes[typeof(DisplayNameAttribute)];
            System.Reflection.FieldInfo fieldToChange = attribute.GetType().GetField("_displayName",
                                             System.Reflection.BindingFlags.NonPublic |
                                             System.Reflection.BindingFlags.Instance);

            if (fieldToChange != null) fieldToChange.SetValue(attribute, displayName);
            property.RefreshPropertygrid();
        }

        public void ToChangeDescription(object obj, string propertyName, string description)
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(obj.GetType())[propertyName];
            DescriptionAttribute attribute = (DescriptionAttribute)
                                          descriptor.Attributes[typeof(DescriptionAttribute)];
            System.Reflection.FieldInfo fieldToChange = attribute.GetType().GetField("description",
                                             System.Reflection.BindingFlags.NonPublic |
                                             System.Reflection.BindingFlags.Instance);

            if (fieldToChange != null) fieldToChange.SetValue(attribute, description);
            property.RefreshPropertygrid();
        }

        public void ToChangeBrowsable(object obj, string propertyName, bool isbrowsable)
        {
            PropertyDescriptor descriptor = TypeDescriptor.GetProperties(obj.GetType())[propertyName];
            BrowsableAttribute attribute = (BrowsableAttribute)
                                          descriptor.Attributes[typeof(BrowsableAttribute)];
            System.Reflection.FieldInfo fieldToChange = attribute.GetType().GetField("browsable",
                                             System.Reflection.BindingFlags.NonPublic |
                                             System.Reflection.BindingFlags.Instance);

            if (fieldToChange != null) fieldToChange.SetValue(attribute, isbrowsable);
            property.RefreshPropertygrid();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            ToChangeDescription(property.SelectedObject, "Name", "Name of the Employee");
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            ToChangeBrowsable(property.SelectedObject, "Name", false);
        }
    }

    public class Employee
    {
        private string name;
        private int age;
        private string designation;

        [Browsable(true)]
        [DisplayName("Name")]
        [Description("Employee Name")]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        public int Age
        {
            get { return age; }
            set { age = value; }
        }

        public string Designation
        {
            get { return designation; }
            set { designation = value; }
        }
        
        public Employee()
        {
            Name = "XXX";
            Age = 24;
            Designation = "Engineer";
        }
    }

}
